==> mutation-0.1.0.tar.gz: lib/src/dispatcher.dart <==
import 'dart:async';

import 'package:rxdart/rxdart.dart';

import 'event.dart';
import 'state.dart';

/// A dispatcher represents a state machine that can
/// [dispatch] events to trigger a mutation of its 
/// [state]
class Dispatcher {
  Dispatcher(State initialState)
      : this._dispatch = StreamController<Event>(sync: true),
        this._completer = Completer<State>(),
        this._mutating = BehaviorSubject(seedValue: false, sync: true),
        this._state =
            BehaviorSubject<State>(seedValue: initialState, sync: true) {
    this._dispatch.stream.listen(_onDispatch);
  }

  /// Indicates whether the current [state] is in the process of mutating for a new state.
  bool get isMutating => this._mutating.value;

  /// Raised whenever the state is mutating.
  Stream<bool> get mutating => this._mutating;

  /// Raised each time the [currentState] has mutated to a new state.
  Stream<State> get state => _state.stream;

  /// Dispatch an event to trigger a mutation of the [currentState].
  ///
  /// If an event is raised while the dispatcher is already [mutating], it is
  /// ignored.
  Sink<Event> get dispatch => _dispatch.sink;

  /// The current state.
  State get currentState => _state.value;

  BehaviorSubject<State> _state;
  BehaviorSubject<bool> _mutating;
  StreamController<Event> _dispatch;
  final Completer<State> _completer;

  /// Indicates that the lifecycle is arrived in one of its
  /// final states (a state with a true value [State.isLast], which
  /// significates that it has no mutations)
  Future complete() => _completer.future;

  /// Indicates whether the [currentState] can mutate from
  /// the given [event].
  bool canMutate(Event event) => this.currentState.canMutate(event);

  void _onDispatch(Event event) async {
    if (!this.isMutating) {
      if (this.currentState.canMutate(event)) {
        try {
          this._mutating.add(true);
          final newState = await this.currentState.mutate(event);
          this._state.add(newState);
          if (newState.isLast) {
            _completer.complete(newState);
          }
        } catch (e) {
          _completer.completeError(e);
        } finally {
          this._mutating.add(false);
        }
      } else {
        /// We return in the the same state
        this._state.add(this.currentState);
      }
    }
  }
}

==> mutation-0.1.1.tar.gz: lib/src/dispatcher.dart <==
import 'dart:async';

import 'package:rxdart/rxdart.dart';

import 'event.dart';
import 'state.dart';

/// A dispatcher represents a state machine that can
/// [dispatch] events to trigger a mutation of its 
/// [state]
class Dispatcher {
  Dispatcher(State initialState)
      : this._dispatch = StreamController<Event>(sync: true),
        this._completer = Completer<State>(),
        this._mutating = BehaviorSubject(seedValue: false, sync: true),
        this._state =
            BehaviorSubject<State>(seedValue: initialState, sync: true) {
    this._dispatchSubscription = this._dispatch.stream.listen(_onDispatch);
  }

  /// Indicates whether the current [state] is in the process of mutating for a new state.
  bool get isMutating => this._mutating.value;

  /// Raised whenever the state is mutating.
  Stream<bool> get mutating => this._mutating;

  /// Raised each time the [currentState] has mutated to a new state.
  Stream<State> get state => _state.stream;

  /// Dispatch an event to trigger a mutation of the [currentState].
  ///
  /// If an event is raised while the dispatcher is already [mutating], it is
  /// ignored.
  Sink<Event> get dispatch => _dispatch.sink;

  /// The current state.
  State get currentState => _state.value;

  BehaviorSubject<State> _state;
  BehaviorSubject<bool> _mutating;
  StreamController<Event> _dispatch;
  StreamSubscription<Event> _dispatchSubscription;
  final Completer<State> _completer;

  /// Indicates that the lifecycle is arrived in one of its
  /// final states (a state with a true value [State.isLast], which
  /// significates that it has no mutations)
  Future complete() => _completer.future;

  /// Indicates whether the [currentState] can mutate from
  /// the given [event].
  bool canMutate(Event event) => this.currentState.canMutate(event);

  /// Dispose the instance by closing all subscriptions.
  void dispose() {
    this._dispatchSubscription.cancel();
    this._mutating.close();
    this._state.close();
    this._dispatch.close();
    if(!_completer.isCompleted) {
      _completer.completeError(Exception("the source dispatcher has been disposed before mutating in its final form"));
    }
  }

  void _onDispatch(Event event) async {
    if (!this.isMutating) {
      if (this.currentState.canMutate(event)) {
        try {
          this._mutating.add(true);
          final newState = await this.currentState.mutate(event);
          this._state.add(newState);
          if (newState.isLast) {
            _completer.complete(newState);
          }
        } catch (e) {
          _completer.completeError(e);
        } finally {
          this._mutating.add(false);
        }
      } else {
        /// We return in the the same state
        this._state.add(this.currentState);
      }
    }
  }
}
